"""
ngm
==========
License: BSD, see LICENSE for more details.
"""
# PYTHON_ARGCOMPLETE_OK
import argcomplete
import argparse
import json
import re

from setuptools_scm import get_version
from tabulate import tabulate


__version__ = get_version(root='..', relative_to=__file__)


def modules_list_installed():
    import os

    # load modules by soname
    modules_json = os.path.dirname(os.path.realpath(__file__)) + '/modules.json'
    with open(modules_json, encoding='utf-8') as json_file:
        modules_data = json.load(json_file)

    modules_dir = '/etc/nginx/modules'
    module_files = [f for f in os.listdir(modules_dir)
                    if f.endswith('.so') and not f.endswith('-debug.so')]
    # TODO add update column (have to consult RPM db)
    all_modules = [['Package Name', 'Feature Summary', 'Load Directive', 'Reference URL']]
    os_codename = None  # e.g. el7
    # get distro
    if module_files:
        import distro
        d = distro.info()
        if d['id'] in ['centos', 'rhel', 'oracle']:
            os_codename = f"el{d['version_parts']['major']}"

    for module_file in module_files:
        package_name = '-'
        feature_summary = ''
        url = ''
        soname = module_file.replace('.so', '')
        if soname in modules_data['by_soname']:
            handle = modules_data['by_soname'][soname]
            package_name = f'nginx-module-{handle}'
            module_data = modules_data['by_handle'][handle]
            feature_summary = shorten_feature_summary(module_data['summary'])
            url = f"https://github.com/{module_data['repo']}"
            if 'ref' in module_data and os_codename in module_data['ref']:
                url = module_data['ref'][os_codename]
        else:
            print(f'{soname} not in modules_data')
        all_modules.append([package_name, feature_summary, f"load_module modules/{module_file};", url])
    print(
        tabulate(all_modules, headers="firstrow", tablefmt='psql')
    )


def shorten_feature_summary(summary):
    module_for = re.compile(re.escape('NGINX module for '), re.IGNORECASE)
    summary = module_for.sub('', summary)
    module_to = re.compile(re.escape('NGINX module to '), re.IGNORECASE)
    summary = module_to.sub('', summary)
    return summary.capitalize()


def module_list_available(format='console'):
    import os

    # load modules by soname
    modules_json = os.path.dirname(os.path.realpath(__file__)) + '/modules.json'
    with open(modules_json) as json_file:
        modules_data = json.load(json_file)

    all_modules = [['ID', 'Feature Summary', 'Load Directive', 'Reference URL']]
    os_codename = None  # e.g. el7
    # get distro
    import distro
    d = distro.info()
    if d['id'] in ['centos', 'rhel', 'oracle']:
        os_codename = f"el{d['version_parts']['major']}"

    for handle, module in modules_data['by_handle'].items():
        sonames = module['soname']
        if not isinstance(sonames, list):
            sonames = [sonames]
        for soname in sonames:
            feature_summary = shorten_feature_summary(module['summary'])
            url = f"https://github.com/{module['repo']}"
            if 'ref' in module and os_codename in module['ref']:
                url = module['ref'][os_codename]
            if format == 'md':
                handle = f"[{handle}]({url})"
                all_modules.append([handle, feature_summary, f"load_module modules/{soname}.so;"])
            else:
                all_modules.append([handle, feature_summary, f"load_module modules/{soname}.so;", url])

    tablefmt = "github" if format == 'md' else 'psql'
    print(
        tabulate(all_modules, headers="firstrow", tablefmt=tablefmt)
    )


def main():
    parser = argparse.ArgumentParser(description='CLI tool for managing NGINX configuration.',
                                     prog='ngm')

    subparsers = parser.add_subparsers(help='Command/area to run', dest="command")

    parser_modules = subparsers.add_parser('modules', help='Manage dynamic NGINX modules')
    modules_subparsers = parser_modules.add_subparsers(dest='action',
                                                       help='An action for managing dynamic NGINX modules')
    # list:
    modules_list_subparser = modules_subparsers.add_parser('list', help='List modules')
    modules_list_subparser.add_argument('kind', nargs='?', default='installed',
                                        choices=['available', 'installed'],
                                        help='Kind of modules to be listed')
    # info:
    # TODO use lastversion for info about latest version
    modules_info_subparser = modules_subparsers.add_parser('info', help='Details about a module')
    modules_info_subparser.add_argument('id', choices=['pagespeed', 'nbr'],
                                        help='ID of a module to query information for')

    parser_sites = subparsers.add_parser('sites', help='Manage NGINX websites')
    sites_subparsers = parser_sites.add_subparsers(dest='action',
                                                   help='An action for managing NGINX websites')
    sites_list_subparser = sites_subparsers.add_parser('list', help='List websites')
    sites_list_subparser.add_argument('kind', nargs='?', default='enabled',
                                      choices=['available', 'enabled'],
                                      help='Kind of websites to be listed')

    parser.add_argument('--version', action='version',
                        version='%(prog)s {version}'.format(version=__version__))


    modules_list_subparser.add_argument('--format', nargs='?', default='console',
                                      choices=['console', 'md'],
                                      help='Console output or Markdown format')

    argcomplete.autocomplete(parser)
    parser_modules.set_defaults(action='list', format='md')
    parser_sites.set_defaults(action='list')
    args = parser.parse_args()

    # import sys
    # f = getattr(sys.modules[__name__], f"{args.command}_{args.action}")
    # f()

    if args.command == 'modules' and args.action == 'list':
        if 'kind' in args and args.kind == 'available':
            module_list_available(args.format)
        else:
            modules_list_installed()
